;====================================================================
;
;  RINT25D.ASM -- a real mode TSR that captures INT 25h and makes a
;       call to the equivalent INT 21h/AX=7305h function under
;       Windows 95 OS Release 2.
;
;  Written by Stephen M. Poole, CET / Majick Attick
;
;  Available for consulting on system-level issues; contact me at
;  Compuserve:71234,3263 (71234.3263@compuserve.com).
;
;  This code is public domain; you may use it in any way you see fit.
;  In doing so, you agree to accept any responsibility for its use,
;  and do so at your own risk.  I make no warranties about the
;  accuracy of this information.
;
;  DATE:            08-07-97
;  LAST MODIFIED:   08-07-97 -- wrote it.
;
;  BUILD WITH any MASM-compatible assembler and linker.
;
;  Set tab stops to (4) for best view.
;
;  Microsoft, Windows, and Windows 95 are registered trademarks.
;====================================================================

.model tiny, c, os_dos          ;Tiny (COM) model

.code

org 5Ch                         ;To save memory, we move everything
                                ; down into the PSP area.

InDOSFlag label dword           ;Address of the INDOS flag
InDOSofst   dw  0
InDOSseg    dw  0

newload label byte

org 100h

start:
   jmp loader                   ;Go load the handler

copypoint:
db "VINT25D "                   ;The VxD looks for this string in
                                ; memory precisely 8 bytes before the
                                ; INT 25h handler.

; -------------------------------------------------------------------
; The INT 25h handler, which maps the call to INT 21h/AX=7305h.  This
; isn't very sophisticated; if it sees an InDOS flag, it bails out.
; I leave it to you to make it more sophisticated, if need be!
;
; ENTRY:    Registers for the INT 25h call
;  EXIT:    Registers returned by DOS, flags still on the stack per
;               convention for INT 25h.
New25:
								;------------------------------------
	push bx                     ;Protect BX and ES
	push es
	les bx, cs:InDOSFlag        ;Get the INDOS Flag address
	cmp byte ptr es:[bx], 1     ;Is it set?
	pop es
	pop bx                      ;(Restore ES and BX)
	stc                         ;(Assume the flag is set)
	jz n25_term                 ;We're done if it is

								;------------------------------------
								;If we get here, DOS is available.
								; Is this an extended INT 25h call?
								; If not, we bail out
	cmp cx, 0FFFFh              ;CX should be -1 if so
	stc                         ;(Assume it isn't)
	mov dl, al                  ;Put the drive number in DL
	mov ax, 207h                ;(Return value, tells the caller to
								; use the extended call)
	jnz n25_term                ;We're done if not

								;------------------------------------
								;OK.  Let's make the call:
	xor si, si                  ;Clear SI for READ
	inc dl                      ;Inc the drive number (it's 1 based)
	mov ax, 7305h               ;DOS Func: Ext_ABSDiskReadWrite
	int 21h

								;------------------------------------
n25_term:                       ;Termination point
	retf                        ;RETF to the caller, leave the flags
								; on the stack by convention

;--------------------------------------------------------------------
; The loader code, which checks for Windows 95 OSR2 and installs this
; handler if it finds it.  (OSR2 currently returns version 7.10, or
; 070Ah, in AX; you may need to change this for subsequent releases.)
; NOTE: This code isn't exactly bulletproof; you may want to add a
; better means of determining a previous load, for example.
loader:
	mov ah, 30h                 ;DOS Func: Get Version
	int 21h
	cmp ax, 0A07h               ;Version 7.10?
	jz l_1                      ;Go ahead if so, else
	jmp WhineAndBail            ; go whine, then bail out

								;------------------------------------
l_1:                            ;Check for a previous installation:

	mov ax, 3525h               ;DOS Func: Get Vector for INT 25h
	int 21h
	sub bx, 8                   ;Move to the string location
	mov di, bx
	mov si, offset copypoint    ;Point to the string
	push si                     ;(Save that offset)
	cld
	mov cx, 8                   ;Compare 8 bytes
	repe cmpsb
	pop si                      ;(Restore SI->copypoint)
	jnz l_2                     ;If not the same, go ahead, else
	mov dx, offset Already      ; tell the operator and git
	mov ah, 9                   ;ANCIENT DOS Func: Print $tring!
	int 21h
	jmp Terminate
								;------------------------------------
l_2:                            ; Move the handler down in memory
								; (SI already points to the block)

	mov di, offset newload      ;Where we'll put it
	mov cx, loader-copypoint    ;# of bytes to copy
	push cs
	pop es                      ;Make sure ES=@code!
	rep movsb
	push di                     ;Save that offset
	mov ah, 34h                 ;DOS Func: Get INDOS Flag
	int 21h
	mov InDOSofst, bx
	mov InDOSseg, es            ;Store the address here
	mov dx, offset newload+8    ;Point to the handler
	mov ax, 2525h               ;DOS Func: Set INT 25h
	int 21h
	mov bx, cs:[2Ch]            ;Get the enviroment address
	mov es, bx                  ;Put it in ES
	mov ah, 49h                 ;DOS Func: Free Memory
	int 21h
	mov dx, offset InstallStr   ;Tell the user
	mov ah, 9
	int 21h
	pop dx                      ;Get the # of bytes copied a moment
	shr dx, 1                   ; ago, divide by 4 to convert to
	shr dx, 1                   ; paragraphs
	shr dx, 1
	shr dx, 1
	inc dx                      ;Add 1 to be safe
	mov ax, 3100h               ;DOS Func: Terminate, Stay Resident
	int 21h

								;------------------------------------
								; If we get here, we didn't have
								; version 7.10.  Whine at the user,
								; then bail out.
WhineAndBail:
	mov dx, offset WhineString  ;Point to the whiney string
	mov ah, 9                   ;ANCIENT DOS Func: Print $tring!
	int 21h

								;------------------------------------
Terminate:                      ;Program termination point for error

	mov ax, 4C01h               ;DOS Func: Terminate (we can HOPE the
	int 21h                     ; user didn't try this on DOS 1!)


;--------------------------------------------------------------------
; Strings:

Already db "The INT 25h handler has already been installed.",0Dh,0Ah,"$"

WhineString db "This program will install a "
    db "handler for INT 25h under Windows 95(tm)",0Dh,0Ah
    db "OEM Release 2 (DOS Version 7.10, Windows "
	db "version 4.00.1111).  You don't",0Dh,0Ah
	db "have this version on your machine; the "
    db "handler has NOT been installed.",0Dh,0Ah,"$"

InstallStr  db "The INT 25h handler for Windows "
	db "95(tm) 0EM R2 has been installed.",0Dh,0Ah,"$"

end start

