;====================================================================
;
;  VINT25D.ASM - VxD that allows older DOS AV programs to work with
;  Windows 95 Release 2 (OSR2).  This works by calling the code in
;  the INT 25h TSR (see RINT25D.ASM) in a nested execution block.
;
;  The initialization code for this VxD doesn't go to a great deal of
;  trouble and expense to determine if the TSR is loaded; it simply
;  looks for the string "ARFINT25" precisely 8 bytes before the INT
;  25h handler in the System VM.  If someone else has loaded an INT
;  25h handler after RINT25D's, this VxD will fail.  Since this code
;  is public domain, I leave it to you to make the needed changes, if
;  you want to distribute this with your product.
;
;  Written by Stephen M. Poole, CET / Majick Attick
;
;  Available for consulting on system-level issues; contact me at
;  Compuserve:71234,3263 (71234.3263@compuserve.com).
;
;  This code is public domain; you may use it in any way you see fit.
;  In doing so, you agree to accept any responsibility for its use,
;  and do so at your own risk.
;
;  DATE:            08-07-97
;  LAST MODIFIED:   08-07-97 -- wrote it.
;
;  BUILD WITH 386-capable MASM and LINK, command lines:
;       ml -coff -c -Cx vint25d.asm
;       link /vxd /nod /def:vint25d.def vint25d.obj
;
;  Set tab stops to (4) for best view.
;
;  Microsoft, Windows, and Windows 95 are registered trademarks.
;====================================================================

.486p                           ;486 protected-mode

.model flat, c

;--------------------------------------------------------------------
;A shortened version of the client registers structure.
ClientRegs struc
	CR_EDI  DD  ?
	CR_ESI  DD  ?
	CR_EBP  DD  ?
	CRPad0  DD  ?
	CR_EBX  DD  ?
	CR_EDX  DD  ?
	CR_ECX  DD  ?
	CR_EAX  DD  ?
	CR_Err  DD  ?
	CR_EIP  DD  ?
	CR_CS   DW  ?
	CRPad1  DW  ?
	CR_EFlags   DD  ?
	CR_ESP  DD  ?
	CR_SS   DW  ?
	CRPad2  DW  ?
	CR_ES   DW  ?
	CRPad3  DW  ?
	CR_DS   DW  ?
	CRPad4  DW  ?
	CR_FS   DW  ?
	CRPad5  DW  ?
	CR_GS   DW  ?
	CRPad6  DW  ?
ClientRegs    ENDS

;--------------------------------------------------------------------
;                L O C K E D    D A T A    A R E A
;
_LDATA SEGMENT DWORD PUBLIC FLAT 'LCODE'
;--------------------------------------------------------------------

;--------------------------------------------------------------------
;              D E V I C E   D E F I N I T I O N   B L O C K
;--------------------------------------------------------------------
public VINT25D_DDB              ;Make this PUBLIC, and EXPORT it in
								; your DEF file
VINT25D_DDB:
	DD  0                       ;FLAT pointer to next VxD (set to 0)
	DW  400h                    ;Version of the DDK used
    DW  0                       ;Device ID (don't care, set to 0)
    DB  4                       ;Major version number (Must be 4!)
    DB  0                       ;Minor version number (Must be 0!)
	DW  0                       ;Status flags (you set these to 0)
astr db "VINT25D "              ;ASCII name
	DD  0B0000000h              ;Init order (after everyone else)
	DD  OFFSET FLAT:Control     ;-> Control procedure
	DD  0                       ;->V86 API procedure
	DD  0                       ;->PM API procedure
	DD  0                       ;CS:IP for V86 API (set to 0)
	DD  0                       ;CS:IP for PM API (set to 0)
	DD  0                       ;-> real-mode reference data
	DD  0                       ;-> VxD-callable service table
	DD  0                       ;# entries in the above table
	DD  0                       ;FLAT ptr to the Win32 service table
	DB  "verP"                  ;Set to ASCII "Prev" by the DDK tools
	DD  50h                     ;Size of this struct in bytes (50h)
	DB  "1vsR"                  ;Set by the DDK tools to ASCII
	DB  "2vsR"                  ;  "Rsv1", "Rsv2", and "Rsv3"
	DB  "3vsR"                  ;(And if you don't do this, Windows
								; will refuse to load your VxD,
								; insisting that you used a "Beta" of
								; the DDK!)

V86Offset   dd  0               ;Segment:Offset of the TSR's INT 25h
V86Segment  dd  0               ; handler in memory

_LDATA ENDS

;--------------------------------------------------------------------
;                L O C K E D    C O D E    A R E A
;
_LTEXT      SEGMENT DWORD PUBLIC FLAT 'LCODE'
;--------------------------------------------------------------------

;--------------------------------------------------------------------
; The Control procedure.  The only messages we capture are Device
; Init and Exit.
;
; ENTRY:    EAX has the message number
;           EBX has the current VM handle
;           EBP -> ClientRegs structure for the current VM
;  EXIT:    Carry cleared to indicate success.
Control proc near
	cmp eax, 1                      ;Device Initialization?
	jnz c_1                         ;Go ahead if not, else

									;--------------------------------
									; Set up our INT 25h V86 handler:

	cld                             ;Set for INC on STOS/MOVS

    mov esi, 25h*4                  ;Point to the vector in the IVT
	xor ecx, ecx                    ;Clear ECX
	mov cx, word ptr [esi]          ;Get the offset, store it here
	mov V86Offset, ecx
	mov cx, word ptr [esi+2]        ;Get the segment, store it here
	mov V86Segment, ecx             ;Store it here
	shl ecx, 4                      ;Convert segment to an offset
	add ecx, V86Offset              ;Add the byte offset
	sub ecx, 8                      ;Subtract 8, point to the string
    mov esi, ecx                    ; in memory (if our TSR's there!)
	mov edi, OFFSET FLAT:astr       ;Point to our comparison string
	mov ecx, 8                      ;We'll compare 8 bytes
	cld
	repe cmpsb
	jz c_25OK                       ;Go ahead if we found our handler
	stc                             ;Otherwise, tell the VMM we can't
	jmp c_end                       ; load and exit

c_25OK:
	mov eax, 25h                    ;We want INT 25h
	mov esi, OFFSET FLAT:New25      ;Point to the handler
	int 20h                         ;VMM Func: Hook V86 Int Chain
	dd 00010041h
	jmp c_end

c_1:
	cmp eax, 5                      ;System Exit?
	clc                             ;(Go ahead and clear the carry)
	jnz c_end                       ;We're done if not

	mov eax, 25h                    ;Otherwise, unhook INT 25h
	mov esi, OFFSET FLAT:New25
	int 20h                         ;VMM Func: Unhook V86 Int Chain
	dd 00010018h
	clc                             ;Clear the carry regardless
c_end:
	ret
Control endp

;--------------------------------------------------------------------
; This is the INT 25h handler, which calls the handler in the RINT25D
; TSR in a nested execution block.
;
; ENTRY:    Client Registers as set for the INT 25h call (CR_CX
;           must have -1 for the new style call)
;  EXIT:    Client Registers and flags as returned by the INT 25h
;           call, with flags still on the stack (as expected)
New25 proc near
	int 20h                         ;VMM Func: Begin Nested Execution
	dd 00010082h
									;Now let's make a FAR call to the
									; TSR's INT 25h handler:
	mov ecx, V86Segment             ;Put the segment in ECX
	mov edx, V86Offset              ;Put the offset in ECX
	int 20h                         ;VMM Func: Simulate Far Call
	dd 00010048h

	int 20h                         ;VMM Func: Resume Execution
    dd 00010085h                    ;(This is what actually causes
                                    ; the FAR CALL to happen)
	int 20h                         ;VMM Func: End Nested Execution
	dd 00010086h

                                    ;Push in the flags, per
                                    ; convention for INT 25h
	mov ax, word ptr [ebp].ClientRegs.CR_EFlags
	int 20h                         ;VMM Func: Simulate Push in VM
	dd 0001004Dh
	clc                             ;Tell the VMM we handled the
	ret                             ; call
New25 endp

_LTEXT      ENDS

end
